# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import TYPE_CHECKING, Any, Dict, Mapping, cast
from typing_extensions import Self, Literal, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
    not_given,
)
from ._utils import is_given, get_async_library
from ._compat import cached_property
from ._version import __version__
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import APIStatusError, SGPClientError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)

if TYPE_CHECKING:
    from .resources import (
        chat,
        build,
        files,
        spans,
        models,
        datasets,
        inference,
        questions,
        responses,
        completions,
        credentials,
        evaluations,
        dataset_items,
        vector_stores,
        evaluation_items,
        span_assessments,
        evaluation_dashboards,
    )
    from .resources.build import BuildResource, AsyncBuildResource
    from .resources.spans import SpansResource, AsyncSpansResource
    from .resources.models import ModelsResource, AsyncModelsResource
    from .resources.datasets import DatasetsResource, AsyncDatasetsResource
    from .resources.chat.chat import ChatResource, AsyncChatResource
    from .resources.inference import InferenceResource, AsyncInferenceResource
    from .resources.questions import QuestionsResource, AsyncQuestionsResource
    from .resources.responses import ResponsesResource, AsyncResponsesResource
    from .resources.completions import CompletionsResource, AsyncCompletionsResource
    from .resources.credentials import CredentialsResource, AsyncCredentialsResource
    from .resources.evaluations import EvaluationsResource, AsyncEvaluationsResource
    from .resources.files.files import FilesResource, AsyncFilesResource
    from .resources.dataset_items import DatasetItemsResource, AsyncDatasetItemsResource
    from .resources.vector_stores import VectorStoresResource, AsyncVectorStoresResource
    from .resources.evaluation_items import EvaluationItemsResource, AsyncEvaluationItemsResource
    from .resources.span_assessments import SpanAssessmentsResource, AsyncSpanAssessmentsResource
    from .resources.evaluation_dashboards.evaluation_dashboards import (
        EvaluationDashboardsResource,
        AsyncEvaluationDashboardsResource,
    )

__all__ = [
    "ENVIRONMENTS",
    "Timeout",
    "Transport",
    "ProxiesTypes",
    "RequestOptions",
    "SGPClient",
    "AsyncSGPClient",
    "Client",
    "AsyncClient",
]

ENVIRONMENTS: Dict[str, str] = {
    "production": "https://api.egp.scale.com",
    "development": "http://127.0.0.1:5003/public",
}


class SGPClient(SyncAPIClient):
    # client options
    api_key: str
    account_id: str

    _environment: Literal["production", "development"] | NotGiven

    def __init__(
        self,
        *,
        api_key: str | None = None,
        account_id: str | None = None,
        environment: Literal["production", "development"] | NotGiven = not_given,
        base_url: str | httpx.URL | None | NotGiven = not_given,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous SGPClient client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `api_key` from `SGP_API_KEY`
        - `account_id` from `SGP_ACCOUNT_ID`
        """
        if api_key is None:
            api_key = os.environ.get("SGP_API_KEY")
        if api_key is None:
            raise SGPClientError(
                "The api_key client option must be set either by passing api_key to the client or by setting the SGP_API_KEY environment variable"
            )
        self.api_key = api_key

        if account_id is None:
            account_id = os.environ.get("SGP_ACCOUNT_ID")
        if account_id is None:
            raise SGPClientError(
                "The account_id client option must be set either by passing account_id to the client or by setting the SGP_ACCOUNT_ID environment variable"
            )
        self.account_id = account_id

        self._environment = environment

        base_url_env = os.environ.get("SGP_CLIENT_BASE_URL")
        if is_given(base_url) and base_url is not None:
            # cast required because mypy doesn't understand the type narrowing
            base_url = cast("str | httpx.URL", base_url)  # pyright: ignore[reportUnnecessaryCast]
        elif is_given(environment):
            if base_url_env and base_url is not None:
                raise ValueError(
                    "Ambiguous URL; The `SGP_CLIENT_BASE_URL` env var and the `environment` argument are given. If you want to use the environment, you must pass base_url=None",
                )

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc
        elif base_url_env is not None:
            base_url = base_url_env
        else:
            self._environment = environment = "production"

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self._default_stream_cls = Stream

    @cached_property
    def responses(self) -> ResponsesResource:
        from .resources.responses import ResponsesResource

        return ResponsesResource(self)

    @cached_property
    def completions(self) -> CompletionsResource:
        from .resources.completions import CompletionsResource

        return CompletionsResource(self)

    @cached_property
    def chat(self) -> ChatResource:
        from .resources.chat import ChatResource

        return ChatResource(self)

    @cached_property
    def inference(self) -> InferenceResource:
        from .resources.inference import InferenceResource

        return InferenceResource(self)

    @cached_property
    def questions(self) -> QuestionsResource:
        from .resources.questions import QuestionsResource

        return QuestionsResource(self)

    @cached_property
    def files(self) -> FilesResource:
        from .resources.files import FilesResource

        return FilesResource(self)

    @cached_property
    def models(self) -> ModelsResource:
        from .resources.models import ModelsResource

        return ModelsResource(self)

    @cached_property
    def datasets(self) -> DatasetsResource:
        from .resources.datasets import DatasetsResource

        return DatasetsResource(self)

    @cached_property
    def dataset_items(self) -> DatasetItemsResource:
        from .resources.dataset_items import DatasetItemsResource

        return DatasetItemsResource(self)

    @cached_property
    def evaluations(self) -> EvaluationsResource:
        from .resources.evaluations import EvaluationsResource

        return EvaluationsResource(self)

    @cached_property
    def evaluation_items(self) -> EvaluationItemsResource:
        from .resources.evaluation_items import EvaluationItemsResource

        return EvaluationItemsResource(self)

    @cached_property
    def evaluation_dashboards(self) -> EvaluationDashboardsResource:
        from .resources.evaluation_dashboards import EvaluationDashboardsResource

        return EvaluationDashboardsResource(self)

    @cached_property
    def spans(self) -> SpansResource:
        from .resources.spans import SpansResource

        return SpansResource(self)

    @cached_property
    def span_assessments(self) -> SpanAssessmentsResource:
        from .resources.span_assessments import SpanAssessmentsResource

        return SpanAssessmentsResource(self)

    @cached_property
    def credentials(self) -> CredentialsResource:
        from .resources.credentials import CredentialsResource

        return CredentialsResource(self)

    @cached_property
    def build(self) -> BuildResource:
        from .resources.build import BuildResource

        return BuildResource(self)

    @cached_property
    def vector_stores(self) -> VectorStoresResource:
        from .resources.vector_stores import VectorStoresResource

        return VectorStoresResource(self)

    @cached_property
    def with_raw_response(self) -> SGPClientWithRawResponse:
        return SGPClientWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SGPClientWithStreamedResponse:
        return SGPClientWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"x-api-key": api_key}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            "x-selected-account-id": self.account_id,
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        account_id: str | None = None,
        environment: Literal["production", "development"] | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            account_id=account_id or self.account_id,
            base_url=base_url or self.base_url,
            environment=environment or self._environment,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncSGPClient(AsyncAPIClient):
    # client options
    api_key: str
    account_id: str

    _environment: Literal["production", "development"] | NotGiven

    def __init__(
        self,
        *,
        api_key: str | None = None,
        account_id: str | None = None,
        environment: Literal["production", "development"] | NotGiven = not_given,
        base_url: str | httpx.URL | None | NotGiven = not_given,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncSGPClient client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `api_key` from `SGP_API_KEY`
        - `account_id` from `SGP_ACCOUNT_ID`
        """
        if api_key is None:
            api_key = os.environ.get("SGP_API_KEY")
        if api_key is None:
            raise SGPClientError(
                "The api_key client option must be set either by passing api_key to the client or by setting the SGP_API_KEY environment variable"
            )
        self.api_key = api_key

        if account_id is None:
            account_id = os.environ.get("SGP_ACCOUNT_ID")
        if account_id is None:
            raise SGPClientError(
                "The account_id client option must be set either by passing account_id to the client or by setting the SGP_ACCOUNT_ID environment variable"
            )
        self.account_id = account_id

        self._environment = environment

        base_url_env = os.environ.get("SGP_CLIENT_BASE_URL")
        if is_given(base_url) and base_url is not None:
            # cast required because mypy doesn't understand the type narrowing
            base_url = cast("str | httpx.URL", base_url)  # pyright: ignore[reportUnnecessaryCast]
        elif is_given(environment):
            if base_url_env and base_url is not None:
                raise ValueError(
                    "Ambiguous URL; The `SGP_CLIENT_BASE_URL` env var and the `environment` argument are given. If you want to use the environment, you must pass base_url=None",
                )

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc
        elif base_url_env is not None:
            base_url = base_url_env
        else:
            self._environment = environment = "production"

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self._default_stream_cls = AsyncStream

    @cached_property
    def responses(self) -> AsyncResponsesResource:
        from .resources.responses import AsyncResponsesResource

        return AsyncResponsesResource(self)

    @cached_property
    def completions(self) -> AsyncCompletionsResource:
        from .resources.completions import AsyncCompletionsResource

        return AsyncCompletionsResource(self)

    @cached_property
    def chat(self) -> AsyncChatResource:
        from .resources.chat import AsyncChatResource

        return AsyncChatResource(self)

    @cached_property
    def inference(self) -> AsyncInferenceResource:
        from .resources.inference import AsyncInferenceResource

        return AsyncInferenceResource(self)

    @cached_property
    def questions(self) -> AsyncQuestionsResource:
        from .resources.questions import AsyncQuestionsResource

        return AsyncQuestionsResource(self)

    @cached_property
    def files(self) -> AsyncFilesResource:
        from .resources.files import AsyncFilesResource

        return AsyncFilesResource(self)

    @cached_property
    def models(self) -> AsyncModelsResource:
        from .resources.models import AsyncModelsResource

        return AsyncModelsResource(self)

    @cached_property
    def datasets(self) -> AsyncDatasetsResource:
        from .resources.datasets import AsyncDatasetsResource

        return AsyncDatasetsResource(self)

    @cached_property
    def dataset_items(self) -> AsyncDatasetItemsResource:
        from .resources.dataset_items import AsyncDatasetItemsResource

        return AsyncDatasetItemsResource(self)

    @cached_property
    def evaluations(self) -> AsyncEvaluationsResource:
        from .resources.evaluations import AsyncEvaluationsResource

        return AsyncEvaluationsResource(self)

    @cached_property
    def evaluation_items(self) -> AsyncEvaluationItemsResource:
        from .resources.evaluation_items import AsyncEvaluationItemsResource

        return AsyncEvaluationItemsResource(self)

    @cached_property
    def evaluation_dashboards(self) -> AsyncEvaluationDashboardsResource:
        from .resources.evaluation_dashboards import AsyncEvaluationDashboardsResource

        return AsyncEvaluationDashboardsResource(self)

    @cached_property
    def spans(self) -> AsyncSpansResource:
        from .resources.spans import AsyncSpansResource

        return AsyncSpansResource(self)

    @cached_property
    def span_assessments(self) -> AsyncSpanAssessmentsResource:
        from .resources.span_assessments import AsyncSpanAssessmentsResource

        return AsyncSpanAssessmentsResource(self)

    @cached_property
    def credentials(self) -> AsyncCredentialsResource:
        from .resources.credentials import AsyncCredentialsResource

        return AsyncCredentialsResource(self)

    @cached_property
    def build(self) -> AsyncBuildResource:
        from .resources.build import AsyncBuildResource

        return AsyncBuildResource(self)

    @cached_property
    def vector_stores(self) -> AsyncVectorStoresResource:
        from .resources.vector_stores import AsyncVectorStoresResource

        return AsyncVectorStoresResource(self)

    @cached_property
    def with_raw_response(self) -> AsyncSGPClientWithRawResponse:
        return AsyncSGPClientWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSGPClientWithStreamedResponse:
        return AsyncSGPClientWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"x-api-key": api_key}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            "x-selected-account-id": self.account_id,
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        account_id: str | None = None,
        environment: Literal["production", "development"] | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            account_id=account_id or self.account_id,
            base_url=base_url or self.base_url,
            environment=environment or self._environment,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class SGPClientWithRawResponse:
    _client: SGPClient

    def __init__(self, client: SGPClient) -> None:
        self._client = client

    @cached_property
    def responses(self) -> responses.ResponsesResourceWithRawResponse:
        from .resources.responses import ResponsesResourceWithRawResponse

        return ResponsesResourceWithRawResponse(self._client.responses)

    @cached_property
    def completions(self) -> completions.CompletionsResourceWithRawResponse:
        from .resources.completions import CompletionsResourceWithRawResponse

        return CompletionsResourceWithRawResponse(self._client.completions)

    @cached_property
    def chat(self) -> chat.ChatResourceWithRawResponse:
        from .resources.chat import ChatResourceWithRawResponse

        return ChatResourceWithRawResponse(self._client.chat)

    @cached_property
    def inference(self) -> inference.InferenceResourceWithRawResponse:
        from .resources.inference import InferenceResourceWithRawResponse

        return InferenceResourceWithRawResponse(self._client.inference)

    @cached_property
    def questions(self) -> questions.QuestionsResourceWithRawResponse:
        from .resources.questions import QuestionsResourceWithRawResponse

        return QuestionsResourceWithRawResponse(self._client.questions)

    @cached_property
    def files(self) -> files.FilesResourceWithRawResponse:
        from .resources.files import FilesResourceWithRawResponse

        return FilesResourceWithRawResponse(self._client.files)

    @cached_property
    def models(self) -> models.ModelsResourceWithRawResponse:
        from .resources.models import ModelsResourceWithRawResponse

        return ModelsResourceWithRawResponse(self._client.models)

    @cached_property
    def datasets(self) -> datasets.DatasetsResourceWithRawResponse:
        from .resources.datasets import DatasetsResourceWithRawResponse

        return DatasetsResourceWithRawResponse(self._client.datasets)

    @cached_property
    def dataset_items(self) -> dataset_items.DatasetItemsResourceWithRawResponse:
        from .resources.dataset_items import DatasetItemsResourceWithRawResponse

        return DatasetItemsResourceWithRawResponse(self._client.dataset_items)

    @cached_property
    def evaluations(self) -> evaluations.EvaluationsResourceWithRawResponse:
        from .resources.evaluations import EvaluationsResourceWithRawResponse

        return EvaluationsResourceWithRawResponse(self._client.evaluations)

    @cached_property
    def evaluation_items(self) -> evaluation_items.EvaluationItemsResourceWithRawResponse:
        from .resources.evaluation_items import EvaluationItemsResourceWithRawResponse

        return EvaluationItemsResourceWithRawResponse(self._client.evaluation_items)

    @cached_property
    def evaluation_dashboards(self) -> evaluation_dashboards.EvaluationDashboardsResourceWithRawResponse:
        from .resources.evaluation_dashboards import EvaluationDashboardsResourceWithRawResponse

        return EvaluationDashboardsResourceWithRawResponse(self._client.evaluation_dashboards)

    @cached_property
    def spans(self) -> spans.SpansResourceWithRawResponse:
        from .resources.spans import SpansResourceWithRawResponse

        return SpansResourceWithRawResponse(self._client.spans)

    @cached_property
    def span_assessments(self) -> span_assessments.SpanAssessmentsResourceWithRawResponse:
        from .resources.span_assessments import SpanAssessmentsResourceWithRawResponse

        return SpanAssessmentsResourceWithRawResponse(self._client.span_assessments)

    @cached_property
    def credentials(self) -> credentials.CredentialsResourceWithRawResponse:
        from .resources.credentials import CredentialsResourceWithRawResponse

        return CredentialsResourceWithRawResponse(self._client.credentials)

    @cached_property
    def build(self) -> build.BuildResourceWithRawResponse:
        from .resources.build import BuildResourceWithRawResponse

        return BuildResourceWithRawResponse(self._client.build)

    @cached_property
    def vector_stores(self) -> vector_stores.VectorStoresResourceWithRawResponse:
        from .resources.vector_stores import VectorStoresResourceWithRawResponse

        return VectorStoresResourceWithRawResponse(self._client.vector_stores)


class AsyncSGPClientWithRawResponse:
    _client: AsyncSGPClient

    def __init__(self, client: AsyncSGPClient) -> None:
        self._client = client

    @cached_property
    def responses(self) -> responses.AsyncResponsesResourceWithRawResponse:
        from .resources.responses import AsyncResponsesResourceWithRawResponse

        return AsyncResponsesResourceWithRawResponse(self._client.responses)

    @cached_property
    def completions(self) -> completions.AsyncCompletionsResourceWithRawResponse:
        from .resources.completions import AsyncCompletionsResourceWithRawResponse

        return AsyncCompletionsResourceWithRawResponse(self._client.completions)

    @cached_property
    def chat(self) -> chat.AsyncChatResourceWithRawResponse:
        from .resources.chat import AsyncChatResourceWithRawResponse

        return AsyncChatResourceWithRawResponse(self._client.chat)

    @cached_property
    def inference(self) -> inference.AsyncInferenceResourceWithRawResponse:
        from .resources.inference import AsyncInferenceResourceWithRawResponse

        return AsyncInferenceResourceWithRawResponse(self._client.inference)

    @cached_property
    def questions(self) -> questions.AsyncQuestionsResourceWithRawResponse:
        from .resources.questions import AsyncQuestionsResourceWithRawResponse

        return AsyncQuestionsResourceWithRawResponse(self._client.questions)

    @cached_property
    def files(self) -> files.AsyncFilesResourceWithRawResponse:
        from .resources.files import AsyncFilesResourceWithRawResponse

        return AsyncFilesResourceWithRawResponse(self._client.files)

    @cached_property
    def models(self) -> models.AsyncModelsResourceWithRawResponse:
        from .resources.models import AsyncModelsResourceWithRawResponse

        return AsyncModelsResourceWithRawResponse(self._client.models)

    @cached_property
    def datasets(self) -> datasets.AsyncDatasetsResourceWithRawResponse:
        from .resources.datasets import AsyncDatasetsResourceWithRawResponse

        return AsyncDatasetsResourceWithRawResponse(self._client.datasets)

    @cached_property
    def dataset_items(self) -> dataset_items.AsyncDatasetItemsResourceWithRawResponse:
        from .resources.dataset_items import AsyncDatasetItemsResourceWithRawResponse

        return AsyncDatasetItemsResourceWithRawResponse(self._client.dataset_items)

    @cached_property
    def evaluations(self) -> evaluations.AsyncEvaluationsResourceWithRawResponse:
        from .resources.evaluations import AsyncEvaluationsResourceWithRawResponse

        return AsyncEvaluationsResourceWithRawResponse(self._client.evaluations)

    @cached_property
    def evaluation_items(self) -> evaluation_items.AsyncEvaluationItemsResourceWithRawResponse:
        from .resources.evaluation_items import AsyncEvaluationItemsResourceWithRawResponse

        return AsyncEvaluationItemsResourceWithRawResponse(self._client.evaluation_items)

    @cached_property
    def evaluation_dashboards(self) -> evaluation_dashboards.AsyncEvaluationDashboardsResourceWithRawResponse:
        from .resources.evaluation_dashboards import AsyncEvaluationDashboardsResourceWithRawResponse

        return AsyncEvaluationDashboardsResourceWithRawResponse(self._client.evaluation_dashboards)

    @cached_property
    def spans(self) -> spans.AsyncSpansResourceWithRawResponse:
        from .resources.spans import AsyncSpansResourceWithRawResponse

        return AsyncSpansResourceWithRawResponse(self._client.spans)

    @cached_property
    def span_assessments(self) -> span_assessments.AsyncSpanAssessmentsResourceWithRawResponse:
        from .resources.span_assessments import AsyncSpanAssessmentsResourceWithRawResponse

        return AsyncSpanAssessmentsResourceWithRawResponse(self._client.span_assessments)

    @cached_property
    def credentials(self) -> credentials.AsyncCredentialsResourceWithRawResponse:
        from .resources.credentials import AsyncCredentialsResourceWithRawResponse

        return AsyncCredentialsResourceWithRawResponse(self._client.credentials)

    @cached_property
    def build(self) -> build.AsyncBuildResourceWithRawResponse:
        from .resources.build import AsyncBuildResourceWithRawResponse

        return AsyncBuildResourceWithRawResponse(self._client.build)

    @cached_property
    def vector_stores(self) -> vector_stores.AsyncVectorStoresResourceWithRawResponse:
        from .resources.vector_stores import AsyncVectorStoresResourceWithRawResponse

        return AsyncVectorStoresResourceWithRawResponse(self._client.vector_stores)


class SGPClientWithStreamedResponse:
    _client: SGPClient

    def __init__(self, client: SGPClient) -> None:
        self._client = client

    @cached_property
    def responses(self) -> responses.ResponsesResourceWithStreamingResponse:
        from .resources.responses import ResponsesResourceWithStreamingResponse

        return ResponsesResourceWithStreamingResponse(self._client.responses)

    @cached_property
    def completions(self) -> completions.CompletionsResourceWithStreamingResponse:
        from .resources.completions import CompletionsResourceWithStreamingResponse

        return CompletionsResourceWithStreamingResponse(self._client.completions)

    @cached_property
    def chat(self) -> chat.ChatResourceWithStreamingResponse:
        from .resources.chat import ChatResourceWithStreamingResponse

        return ChatResourceWithStreamingResponse(self._client.chat)

    @cached_property
    def inference(self) -> inference.InferenceResourceWithStreamingResponse:
        from .resources.inference import InferenceResourceWithStreamingResponse

        return InferenceResourceWithStreamingResponse(self._client.inference)

    @cached_property
    def questions(self) -> questions.QuestionsResourceWithStreamingResponse:
        from .resources.questions import QuestionsResourceWithStreamingResponse

        return QuestionsResourceWithStreamingResponse(self._client.questions)

    @cached_property
    def files(self) -> files.FilesResourceWithStreamingResponse:
        from .resources.files import FilesResourceWithStreamingResponse

        return FilesResourceWithStreamingResponse(self._client.files)

    @cached_property
    def models(self) -> models.ModelsResourceWithStreamingResponse:
        from .resources.models import ModelsResourceWithStreamingResponse

        return ModelsResourceWithStreamingResponse(self._client.models)

    @cached_property
    def datasets(self) -> datasets.DatasetsResourceWithStreamingResponse:
        from .resources.datasets import DatasetsResourceWithStreamingResponse

        return DatasetsResourceWithStreamingResponse(self._client.datasets)

    @cached_property
    def dataset_items(self) -> dataset_items.DatasetItemsResourceWithStreamingResponse:
        from .resources.dataset_items import DatasetItemsResourceWithStreamingResponse

        return DatasetItemsResourceWithStreamingResponse(self._client.dataset_items)

    @cached_property
    def evaluations(self) -> evaluations.EvaluationsResourceWithStreamingResponse:
        from .resources.evaluations import EvaluationsResourceWithStreamingResponse

        return EvaluationsResourceWithStreamingResponse(self._client.evaluations)

    @cached_property
    def evaluation_items(self) -> evaluation_items.EvaluationItemsResourceWithStreamingResponse:
        from .resources.evaluation_items import EvaluationItemsResourceWithStreamingResponse

        return EvaluationItemsResourceWithStreamingResponse(self._client.evaluation_items)

    @cached_property
    def evaluation_dashboards(self) -> evaluation_dashboards.EvaluationDashboardsResourceWithStreamingResponse:
        from .resources.evaluation_dashboards import EvaluationDashboardsResourceWithStreamingResponse

        return EvaluationDashboardsResourceWithStreamingResponse(self._client.evaluation_dashboards)

    @cached_property
    def spans(self) -> spans.SpansResourceWithStreamingResponse:
        from .resources.spans import SpansResourceWithStreamingResponse

        return SpansResourceWithStreamingResponse(self._client.spans)

    @cached_property
    def span_assessments(self) -> span_assessments.SpanAssessmentsResourceWithStreamingResponse:
        from .resources.span_assessments import SpanAssessmentsResourceWithStreamingResponse

        return SpanAssessmentsResourceWithStreamingResponse(self._client.span_assessments)

    @cached_property
    def credentials(self) -> credentials.CredentialsResourceWithStreamingResponse:
        from .resources.credentials import CredentialsResourceWithStreamingResponse

        return CredentialsResourceWithStreamingResponse(self._client.credentials)

    @cached_property
    def build(self) -> build.BuildResourceWithStreamingResponse:
        from .resources.build import BuildResourceWithStreamingResponse

        return BuildResourceWithStreamingResponse(self._client.build)

    @cached_property
    def vector_stores(self) -> vector_stores.VectorStoresResourceWithStreamingResponse:
        from .resources.vector_stores import VectorStoresResourceWithStreamingResponse

        return VectorStoresResourceWithStreamingResponse(self._client.vector_stores)


class AsyncSGPClientWithStreamedResponse:
    _client: AsyncSGPClient

    def __init__(self, client: AsyncSGPClient) -> None:
        self._client = client

    @cached_property
    def responses(self) -> responses.AsyncResponsesResourceWithStreamingResponse:
        from .resources.responses import AsyncResponsesResourceWithStreamingResponse

        return AsyncResponsesResourceWithStreamingResponse(self._client.responses)

    @cached_property
    def completions(self) -> completions.AsyncCompletionsResourceWithStreamingResponse:
        from .resources.completions import AsyncCompletionsResourceWithStreamingResponse

        return AsyncCompletionsResourceWithStreamingResponse(self._client.completions)

    @cached_property
    def chat(self) -> chat.AsyncChatResourceWithStreamingResponse:
        from .resources.chat import AsyncChatResourceWithStreamingResponse

        return AsyncChatResourceWithStreamingResponse(self._client.chat)

    @cached_property
    def inference(self) -> inference.AsyncInferenceResourceWithStreamingResponse:
        from .resources.inference import AsyncInferenceResourceWithStreamingResponse

        return AsyncInferenceResourceWithStreamingResponse(self._client.inference)

    @cached_property
    def questions(self) -> questions.AsyncQuestionsResourceWithStreamingResponse:
        from .resources.questions import AsyncQuestionsResourceWithStreamingResponse

        return AsyncQuestionsResourceWithStreamingResponse(self._client.questions)

    @cached_property
    def files(self) -> files.AsyncFilesResourceWithStreamingResponse:
        from .resources.files import AsyncFilesResourceWithStreamingResponse

        return AsyncFilesResourceWithStreamingResponse(self._client.files)

    @cached_property
    def models(self) -> models.AsyncModelsResourceWithStreamingResponse:
        from .resources.models import AsyncModelsResourceWithStreamingResponse

        return AsyncModelsResourceWithStreamingResponse(self._client.models)

    @cached_property
    def datasets(self) -> datasets.AsyncDatasetsResourceWithStreamingResponse:
        from .resources.datasets import AsyncDatasetsResourceWithStreamingResponse

        return AsyncDatasetsResourceWithStreamingResponse(self._client.datasets)

    @cached_property
    def dataset_items(self) -> dataset_items.AsyncDatasetItemsResourceWithStreamingResponse:
        from .resources.dataset_items import AsyncDatasetItemsResourceWithStreamingResponse

        return AsyncDatasetItemsResourceWithStreamingResponse(self._client.dataset_items)

    @cached_property
    def evaluations(self) -> evaluations.AsyncEvaluationsResourceWithStreamingResponse:
        from .resources.evaluations import AsyncEvaluationsResourceWithStreamingResponse

        return AsyncEvaluationsResourceWithStreamingResponse(self._client.evaluations)

    @cached_property
    def evaluation_items(self) -> evaluation_items.AsyncEvaluationItemsResourceWithStreamingResponse:
        from .resources.evaluation_items import AsyncEvaluationItemsResourceWithStreamingResponse

        return AsyncEvaluationItemsResourceWithStreamingResponse(self._client.evaluation_items)

    @cached_property
    def evaluation_dashboards(self) -> evaluation_dashboards.AsyncEvaluationDashboardsResourceWithStreamingResponse:
        from .resources.evaluation_dashboards import AsyncEvaluationDashboardsResourceWithStreamingResponse

        return AsyncEvaluationDashboardsResourceWithStreamingResponse(self._client.evaluation_dashboards)

    @cached_property
    def spans(self) -> spans.AsyncSpansResourceWithStreamingResponse:
        from .resources.spans import AsyncSpansResourceWithStreamingResponse

        return AsyncSpansResourceWithStreamingResponse(self._client.spans)

    @cached_property
    def span_assessments(self) -> span_assessments.AsyncSpanAssessmentsResourceWithStreamingResponse:
        from .resources.span_assessments import AsyncSpanAssessmentsResourceWithStreamingResponse

        return AsyncSpanAssessmentsResourceWithStreamingResponse(self._client.span_assessments)

    @cached_property
    def credentials(self) -> credentials.AsyncCredentialsResourceWithStreamingResponse:
        from .resources.credentials import AsyncCredentialsResourceWithStreamingResponse

        return AsyncCredentialsResourceWithStreamingResponse(self._client.credentials)

    @cached_property
    def build(self) -> build.AsyncBuildResourceWithStreamingResponse:
        from .resources.build import AsyncBuildResourceWithStreamingResponse

        return AsyncBuildResourceWithStreamingResponse(self._client.build)

    @cached_property
    def vector_stores(self) -> vector_stores.AsyncVectorStoresResourceWithStreamingResponse:
        from .resources.vector_stores import AsyncVectorStoresResourceWithStreamingResponse

        return AsyncVectorStoresResourceWithStreamingResponse(self._client.vector_stores)


Client = SGPClient

AsyncClient = AsyncSGPClient



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, TypedDict

__all__ = ["EvaluationDashboardRetrieveParams"]


class EvaluationDashboardRetrieveParams(TypedDict, total=False):
    include_archived: bool

    views: List[Literal["widgets", "widget_results"]]
    """Optional relationships to include: 'widgets', 'widget_results'"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, Dict, cast

import httpx

from ..types import inference_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.inference_create_response import InferenceCreateResponse

__all__ = ["InferenceResource", "AsyncInferenceResource"]


class InferenceResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> InferenceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python-beta#accessing-raw-response-data-eg-headers
        """
        return InferenceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> InferenceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python-beta#with_streaming_response
        """
        return InferenceResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        model: str,
        args: Dict[str, object] | Omit = omit,
        inference_configuration: inference_create_params.InferenceConfiguration | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InferenceCreateResponse:
        """Generic Inference

        Args:
          model: model specified as `vendor/name` (ex.

        openai/gpt-5)

          args: Arguments passed into model

          inference_configuration: Vendor specific configuration

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return cast(
            InferenceCreateResponse,
            self._post(
                "/v5/inference",
                body=maybe_transform(
                    {
                        "model": model,
                        "args": args,
                        "inference_configuration": inference_configuration,
                    },
                    inference_create_params.InferenceCreateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, InferenceCreateResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class AsyncInferenceResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncInferenceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python-beta#accessing-raw-response-data-eg-headers
        """
        return AsyncInferenceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncInferenceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python-beta#with_streaming_response
        """
        return AsyncInferenceResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        model: str,
        args: Dict[str, object] | Omit = omit,
        inference_configuration: inference_create_params.InferenceConfiguration | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InferenceCreateResponse:
        """Generic Inference

        Args:
          model: model specified as `vendor/name` (ex.

        openai/gpt-5)

          args: Arguments passed into model

          inference_configuration: Vendor specific configuration

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return cast(
            InferenceCreateResponse,
            await self._post(
                "/v5/inference",
                body=await async_maybe_transform(
                    {
                        "model": model,
                        "args": args,
                        "inference_configuration": inference_configuration,
                    },
                    inference_create_params.InferenceCreateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, InferenceCreateResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class InferenceResourceWithRawResponse:
    def __init__(self, inference: InferenceResource) -> None:
        self._inference = inference

        self.create = to_raw_response_wrapper(
            inference.create,
        )


class AsyncInferenceResourceWithRawResponse:
    def __init__(self, inference: AsyncInferenceResource) -> None:
        self._inference = inference

        self.create = async_to_raw_response_wrapper(
            inference.create,
        )


class InferenceResourceWithStreamingResponse:
    def __init__(self, inference: InferenceResource) -> None:
        self._inference = inference

        self.create = to_streamed_response_wrapper(
            inference.create,
        )


class AsyncInferenceResourceWithStreamingResponse:
    def __init__(self, inference: AsyncInferenceResource) -> None:
        self._inference = inference

        self.create = async_to_streamed_response_wrapper(
            inference.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from scale_gp_beta import SGPClient, AsyncSGPClient
from scale_gp_beta.types import (
    BuildListResponse,
    BuildCancelResponse,
    BuildCreateResponse,
    BuildRetrieveResponse,
)
from scale_gp_beta.pagination import SyncCursorPage, AsyncCursorPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestBuild:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: SGPClient) -> None:
        build = client.build.create(
            context_archive=b"raw file contents",
            image_name="image_name",
        )
        assert_matches_type(BuildCreateResponse, build, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: SGPClient) -> None:
        build = client.build.create(
            context_archive=b"raw file contents",
            image_name="image_name",
            build_args="build_args",
            image_tag="image_tag",
        )
        assert_matches_type(BuildCreateResponse, build, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: SGPClient) -> None:
        response = client.build.with_raw_response.create(
            context_archive=b"raw file contents",
            image_name="image_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        build = response.parse()
        assert_matches_type(BuildCreateResponse, build, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: SGPClient) -> None:
        with client.build.with_streaming_response.create(
            context_archive=b"raw file contents",
            image_name="image_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            build = response.parse()
            assert_matches_type(BuildCreateResponse, build, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: SGPClient) -> None:
        build = client.build.retrieve(
            "build_id",
        )
        assert_matches_type(BuildRetrieveResponse, build, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: SGPClient) -> None:
        response = client.build.with_raw_response.retrieve(
            "build_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        build = response.parse()
        assert_matches_type(BuildRetrieveResponse, build, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: SGPClient) -> None:
        with client.build.with_streaming_response.retrieve(
            "build_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            build = response.parse()
            assert_matches_type(BuildRetrieveResponse, build, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `build_id` but received ''"):
            client.build.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_list(self, client: SGPClient) -> None:
        build = client.build.list()
        assert_matches_type(SyncCursorPage[BuildListResponse], build, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: SGPClient) -> None:
        build = client.build.list(
            ending_before="ending_before",
            limit=1,
            sort_by="sort_by",
            sort_order="asc",
            starting_after="starting_after",
        )
        assert_matches_type(SyncCursorPage[BuildListResponse], build, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: SGPClient) -> None:
        response = client.build.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        build = response.parse()
        assert_matches_type(SyncCursorPage[BuildListResponse], build, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: SGPClient) -> None:
        with client.build.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            build = response.parse()
            assert_matches_type(SyncCursorPage[BuildListResponse], build, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_cancel(self, client: SGPClient) -> None:
        build = client.build.cancel(
            "build_id",
        )
        assert_matches_type(BuildCancelResponse, build, path=["response"])

    @parametrize
    def test_raw_response_cancel(self, client: SGPClient) -> None:
        response = client.build.with_raw_response.cancel(
            "build_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        build = response.parse()
        assert_matches_type(BuildCancelResponse, build, path=["response"])

    @parametrize
    def test_streaming_response_cancel(self, client: SGPClient) -> None:
        with client.build.with_streaming_response.cancel(
            "build_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            build = response.parse()
            assert_matches_type(BuildCancelResponse, build, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_cancel(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `build_id` but received ''"):
            client.build.with_raw_response.cancel(
                "",
            )

    @parametrize
    def test_method_logs(self, client: SGPClient) -> None:
        build_stream = client.build.logs(
            "build_id",
        )
        build_stream.response.close()

    @parametrize
    def test_raw_response_logs(self, client: SGPClient) -> None:
        response = client.build.with_raw_response.logs(
            "build_id",
        )

        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        stream = response.parse()
        stream.close()

    @parametrize
    def test_streaming_response_logs(self, client: SGPClient) -> None:
        with client.build.with_streaming_response.logs(
            "build_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            stream = response.parse()
            stream.close()

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_logs(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `build_id` but received ''"):
            client.build.with_raw_response.logs(
                "",
            )


class TestAsyncBuild:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncSGPClient) -> None:
        build = await async_client.build.create(
            context_archive=b"raw file contents",
            image_name="image_name",
        )
        assert_matches_type(BuildCreateResponse, build, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncSGPClient) -> None:
        build = await async_client.build.create(
            context_archive=b"raw file contents",
            image_name="image_name",
            build_args="build_args",
            image_tag="image_tag",
        )
        assert_matches_type(BuildCreateResponse, build, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.build.with_raw_response.create(
            context_archive=b"raw file contents",
            image_name="image_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        build = await response.parse()
        assert_matches_type(BuildCreateResponse, build, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncSGPClient) -> None:
        async with async_client.build.with_streaming_response.create(
            context_archive=b"raw file contents",
            image_name="image_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            build = await response.parse()
            assert_matches_type(BuildCreateResponse, build, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncSGPClient) -> None:
        build = await async_client.build.retrieve(
            "build_id",
        )
        assert_matches_type(BuildRetrieveResponse, build, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.build.with_raw_response.retrieve(
            "build_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        build = await response.parse()
        assert_matches_type(BuildRetrieveResponse, build, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        async with async_client.build.with_streaming_response.retrieve(
            "build_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            build = await response.parse()
            assert_matches_type(BuildRetrieveResponse, build, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `build_id` but received ''"):
            await async_client.build.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncSGPClient) -> None:
        build = await async_client.build.list()
        assert_matches_type(AsyncCursorPage[BuildListResponse], build, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncSGPClient) -> None:
        build = await async_client.build.list(
            ending_before="ending_before",
            limit=1,
            sort_by="sort_by",
            sort_order="asc",
            starting_after="starting_after",
        )
        assert_matches_type(AsyncCursorPage[BuildListResponse], build, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.build.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        build = await response.parse()
        assert_matches_type(AsyncCursorPage[BuildListResponse], build, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncSGPClient) -> None:
        async with async_client.build.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            build = await response.parse()
            assert_matches_type(AsyncCursorPage[BuildListResponse], build, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_cancel(self, async_client: AsyncSGPClient) -> None:
        build = await async_client.build.cancel(
            "build_id",
        )
        assert_matches_type(BuildCancelResponse, build, path=["response"])

    @parametrize
    async def test_raw_response_cancel(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.build.with_raw_response.cancel(
            "build_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        build = await response.parse()
        assert_matches_type(BuildCancelResponse, build, path=["response"])

    @parametrize
    async def test_streaming_response_cancel(self, async_client: AsyncSGPClient) -> None:
        async with async_client.build.with_streaming_response.cancel(
            "build_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            build = await response.parse()
            assert_matches_type(BuildCancelResponse, build, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_cancel(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `build_id` but received ''"):
            await async_client.build.with_raw_response.cancel(
                "",
            )

    @parametrize
    async def test_method_logs(self, async_client: AsyncSGPClient) -> None:
        build_stream = await async_client.build.logs(
            "build_id",
        )
        await build_stream.response.aclose()

    @parametrize
    async def test_raw_response_logs(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.build.with_raw_response.logs(
            "build_id",
        )

        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        stream = await response.parse()
        await stream.close()

    @parametrize
    async def test_streaming_response_logs(self, async_client: AsyncSGPClient) -> None:
        async with async_client.build.with_streaming_response.logs(
            "build_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            stream = await response.parse()
            await stream.close()

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_logs(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `build_id` but received ''"):
            await async_client.build.with_raw_response.logs(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["StreamChunk"]


class StreamChunk(BaseModel):
    """A single log line from the build process."""

    line: str
    """The log line content"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .chat import (
    ChatResource,
    AsyncChatResource,
    ChatResourceWithRawResponse,
    AsyncChatResourceWithRawResponse,
    ChatResourceWithStreamingResponse,
    AsyncChatResourceWithStreamingResponse,
)
from .build import (
    BuildResource,
    AsyncBuildResource,
    BuildResourceWithRawResponse,
    AsyncBuildResourceWithRawResponse,
    BuildResourceWithStreamingResponse,
    AsyncBuildResourceWithStreamingResponse,
)
from .files import (
    FilesResource,
    AsyncFilesResource,
    FilesResourceWithRawResponse,
    AsyncFilesResourceWithRawResponse,
    FilesResourceWithStreamingResponse,
    AsyncFilesResourceWithStreamingResponse,
)
from .spans import (
    SpansResource,
    AsyncSpansResource,
    SpansResourceWithRawResponse,
    AsyncSpansResourceWithRawResponse,
    SpansResourceWithStreamingResponse,
    AsyncSpansResourceWithStreamingResponse,
)
from .models import (
    ModelsResource,
    AsyncModelsResource,
    ModelsResourceWithRawResponse,
    AsyncModelsResourceWithRawResponse,
    ModelsResourceWithStreamingResponse,
    AsyncModelsResourceWithStreamingResponse,
)
from .datasets import (
    DatasetsResource,
    AsyncDatasetsResource,
    DatasetsResourceWithRawResponse,
    AsyncDatasetsResourceWithRawResponse,
    DatasetsResourceWithStreamingResponse,
    AsyncDatasetsResourceWithStreamingResponse,
)
from .inference import (
    InferenceResource,
    AsyncInferenceResource,
    InferenceResourceWithRawResponse,
    AsyncInferenceResourceWithRawResponse,
    InferenceResourceWithStreamingResponse,
    AsyncInferenceResourceWithStreamingResponse,
)
from .questions import (
    QuestionsResource,
    AsyncQuestionsResource,
    QuestionsResourceWithRawResponse,
    AsyncQuestionsResourceWithRawResponse,
    QuestionsResourceWithStreamingResponse,
    AsyncQuestionsResourceWithStreamingResponse,
)
from .responses import (
    ResponsesResource,
    AsyncResponsesResource,
    ResponsesResourceWithRawResponse,
    AsyncResponsesResourceWithRawResponse,
    ResponsesResourceWithStreamingResponse,
    AsyncResponsesResourceWithStreamingResponse,
)
from .completions import (
    CompletionsResource,
    AsyncCompletionsResource,
    CompletionsResourceWithRawResponse,
    AsyncCompletionsResourceWithRawResponse,
    CompletionsResourceWithStreamingResponse,
    AsyncCompletionsResourceWithStreamingResponse,
)
from .credentials import (
    CredentialsResource,
    AsyncCredentialsResource,
    CredentialsResourceWithRawResponse,
    AsyncCredentialsResourceWithRawResponse,
    CredentialsResourceWithStreamingResponse,
    AsyncCredentialsResourceWithStreamingResponse,
)
from .evaluations import (
    EvaluationsResource,
    AsyncEvaluationsResource,
    EvaluationsResourceWithRawResponse,
    AsyncEvaluationsResourceWithRawResponse,
    EvaluationsResourceWithStreamingResponse,
    AsyncEvaluationsResourceWithStreamingResponse,
)
from .dataset_items import (
    DatasetItemsResource,
    AsyncDatasetItemsResource,
    DatasetItemsResourceWithRawResponse,
    AsyncDatasetItemsResourceWithRawResponse,
    DatasetItemsResourceWithStreamingResponse,
    AsyncDatasetItemsResourceWithStreamingResponse,
)
from .vector_stores import (
    VectorStoresResource,
    AsyncVectorStoresResource,
    VectorStoresResourceWithRawResponse,
    AsyncVectorStoresResourceWithRawResponse,
    VectorStoresResourceWithStreamingResponse,
    AsyncVectorStoresResourceWithStreamingResponse,
)
from .evaluation_items import (
    EvaluationItemsResource,
    AsyncEvaluationItemsResource,
    EvaluationItemsResourceWithRawResponse,
    AsyncEvaluationItemsResourceWithRawResponse,
    EvaluationItemsResourceWithStreamingResponse,
    AsyncEvaluationItemsResourceWithStreamingResponse,
)
from .span_assessments import (
    SpanAssessmentsResource,
    AsyncSpanAssessmentsResource,
    SpanAssessmentsResourceWithRawResponse,
    AsyncSpanAssessmentsResourceWithRawResponse,
    SpanAssessmentsResourceWithStreamingResponse,
    AsyncSpanAssessmentsResourceWithStreamingResponse,
)
from .evaluation_dashboards import (
    EvaluationDashboardsResource,
    AsyncEvaluationDashboardsResource,
    EvaluationDashboardsResourceWithRawResponse,
    AsyncEvaluationDashboardsResourceWithRawResponse,
    EvaluationDashboardsResourceWithStreamingResponse,
    AsyncEvaluationDashboardsResourceWithStreamingResponse,
)

__all__ = [
    "ResponsesResource",
    "AsyncResponsesResource",
    "ResponsesResourceWithRawResponse",
    "AsyncResponsesResourceWithRawResponse",
    "ResponsesResourceWithStreamingResponse",
    "AsyncResponsesResourceWithStreamingResponse",
    "CompletionsResource",
    "AsyncCompletionsResource",
    "CompletionsResourceWithRawResponse",
    "AsyncCompletionsResourceWithRawResponse",
    "CompletionsResourceWithStreamingResponse",
    "AsyncCompletionsResourceWithStreamingResponse",
    "ChatResource",
    "AsyncChatResource",
    "ChatResourceWithRawResponse",
    "AsyncChatResourceWithRawResponse",
    "ChatResourceWithStreamingResponse",
    "AsyncChatResourceWithStreamingResponse",
    "InferenceResource",
    "AsyncInferenceResource",
    "InferenceResourceWithRawResponse",
    "AsyncInferenceResourceWithRawResponse",
    "InferenceResourceWithStreamingResponse",
    "AsyncInferenceResourceWithStreamingResponse",
    "QuestionsResource",
    "AsyncQuestionsResource",
    "QuestionsResourceWithRawResponse",
    "AsyncQuestionsResourceWithRawResponse",
    "QuestionsResourceWithStreamingResponse",
    "AsyncQuestionsResourceWithStreamingResponse",
    "FilesResource",
    "AsyncFilesResource",
    "FilesResourceWithRawResponse",
    "AsyncFilesResourceWithRawResponse",
    "FilesResourceWithStreamingResponse",
    "AsyncFilesResourceWithStreamingResponse",
    "ModelsResource",
    "AsyncModelsResource",
    "ModelsResourceWithRawResponse",
    "AsyncModelsResourceWithRawResponse",
    "ModelsResourceWithStreamingResponse",
    "AsyncModelsResourceWithStreamingResponse",
    "DatasetsResource",
    "AsyncDatasetsResource",
    "DatasetsResourceWithRawResponse",
    "AsyncDatasetsResourceWithRawResponse",
    "DatasetsResourceWithStreamingResponse",
    "AsyncDatasetsResourceWithStreamingResponse",
    "DatasetItemsResource",
    "AsyncDatasetItemsResource",
    "DatasetItemsResourceWithRawResponse",
    "AsyncDatasetItemsResourceWithRawResponse",
    "DatasetItemsResourceWithStreamingResponse",
    "AsyncDatasetItemsResourceWithStreamingResponse",
    "EvaluationsResource",
    "AsyncEvaluationsResource",
    "EvaluationsResourceWithRawResponse",
    "AsyncEvaluationsResourceWithRawResponse",
    "EvaluationsResourceWithStreamingResponse",
    "AsyncEvaluationsResourceWithStreamingResponse",
    "EvaluationItemsResource",
    "AsyncEvaluationItemsResource",
    "EvaluationItemsResourceWithRawResponse",
    "AsyncEvaluationItemsResourceWithRawResponse",
    "EvaluationItemsResourceWithStreamingResponse",
    "AsyncEvaluationItemsResourceWithStreamingResponse",
    "EvaluationDashboardsResource",
    "AsyncEvaluationDashboardsResource",
    "EvaluationDashboardsResourceWithRawResponse",
    "AsyncEvaluationDashboardsResourceWithRawResponse",
    "EvaluationDashboardsResourceWithStreamingResponse",
    "AsyncEvaluationDashboardsResourceWithStreamingResponse",
    "SpansResource",
    "AsyncSpansResource",
    "SpansResourceWithRawResponse",
    "AsyncSpansResourceWithRawResponse",
    "SpansResourceWithStreamingResponse",
    "AsyncSpansResourceWithStreamingResponse",
    "SpanAssessmentsResource",
    "AsyncSpanAssessmentsResource",
    "SpanAssessmentsResourceWithRawResponse",
    "AsyncSpanAssessmentsResourceWithRawResponse",
    "SpanAssessmentsResourceWithStreamingResponse",
    "AsyncSpanAssessmentsResourceWithStreamingResponse",
    "CredentialsResource",
    "AsyncCredentialsResource",
    "CredentialsResourceWithRawResponse",
    "AsyncCredentialsResourceWithRawResponse",
    "CredentialsResourceWithStreamingResponse",
    "AsyncCredentialsResourceWithStreamingResponse",
    "BuildResource",
    "AsyncBuildResource",
    "BuildResourceWithRawResponse",
    "AsyncBuildResourceWithRawResponse",
    "BuildResourceWithStreamingResponse",
    "AsyncBuildResourceWithStreamingResponse",
    "VectorStoresResource",
    "AsyncVectorStoresResource",
    "VectorStoresResourceWithRawResponse",
    "AsyncVectorStoresResourceWithRawResponse",
    "VectorStoresResourceWithStreamingResponse",
    "AsyncVectorStoresResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .widget_create_params import WidgetCreateParams as WidgetCreateParams
from .widget_update_params import WidgetUpdateParams as WidgetUpdateParams
from .evaluation_dashboard_widget import EvaluationDashboardWidget as EvaluationDashboardWidget
from .evaluation_dashboard_widget_result import EvaluationDashboardWidgetResult as EvaluationDashboardWidgetResult
from .evaluation_dashboard_widget_with_result import (
    EvaluationDashboardWidgetWithResult as EvaluationDashboardWidgetWithResult,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .._types import SequenceNotStr
from .item_locator import ItemLocator

__all__ = [
    "EvaluationTaskParam",
    "ChatCompletionEvaluationTask",
    "ChatCompletionEvaluationTaskConfiguration",
    "GenericInferenceEvaluationTask",
    "GenericInferenceEvaluationTaskConfiguration",
    "GenericInferenceEvaluationTaskConfigurationInferenceConfiguration",
    "GenericInferenceEvaluationTaskConfigurationInferenceConfigurationLaunchInferenceConfiguration",
    "ApplicationVariantV1EvaluationTask",
    "ApplicationVariantV1EvaluationTaskConfiguration",
    "ApplicationVariantV1EvaluationTaskConfigurationHistoryApplicationRequestResponsePairArray",
    "ApplicationVariantV1EvaluationTaskConfigurationOverrides",
    "ApplicationVariantV1EvaluationTaskConfigurationOverridesAgenticApplicationOverrides",
    "ApplicationVariantV1EvaluationTaskConfigurationOverridesAgenticApplicationOverridesInitialState",
    "ApplicationVariantV1EvaluationTaskConfigurationOverridesAgenticApplicationOverridesPartialTrace",
    "AgentexOutputEvaluationTask",
    "AgentexOutputEvaluationTaskConfiguration",
    "MetricEvaluationTask",
    "MetricEvaluationTaskConfiguration",
    "MetricEvaluationTaskConfigurationBleuScorerConfigWithItemLocator",
    "MetricEvaluationTaskConfigurationMeteorScorerConfigWithItemLocator",
    "MetricEvaluationTaskConfigurationCosineSimilarityScorerConfigWithItemLocator",
    "MetricEvaluationTaskConfigurationF1ScorerConfigWithItemLocator",
    "MetricEvaluationTaskConfigurationRougeScorer1ConfigWithItemLocator",
    "MetricEvaluationTaskConfigurationRougeScorer2ConfigWithItemLocator",
    "MetricEvaluationTaskConfigurationRougeScorerLConfigWithItemLocator",
    "AutoEvaluationQuestionTask",
    "AutoEvaluationQuestionTaskConfiguration",
    "AutoEvaluationGuidedDecodingEvaluationTask",
    "AutoEvaluationGuidedDecodingEvaluationTaskConfiguration",
    "AutoEvaluationGuidedDecodingEvaluationTaskConfigurationAutoEvaluationStructuredOutputTaskRequestWithItemLocator",
    "AutoEvaluationGuidedDecodingEvaluationTaskConfigurationAutoEvaluationGuidedDecodingTaskRequestWithItemLocator",
    "AutoEvaluationGuidedDecodingEvaluationTaskConfigurationAutoEvaluationAgentTaskRequestWithItemLocator",
    "AutoEvaluationGuidedDecodingEvaluationTaskConfigurationAutoEvaluationAgentTaskRequestWithItemLocatorDesignatedTo",
    "AutoEvaluationGuidedDecodingEvaluationTaskConfigurationAutoEvaluationAgentTaskRequestWithItemLocatorDesignatedToApeAgent",
    "AutoEvaluationGuidedDecodingEvaluationTaskConfigurationAutoEvaluationAgentTaskRequestWithItemLocatorDesignatedToApeAgentConfig",
    "AutoEvaluationGuidedDecodingEvaluationTaskConfigurationAutoEvaluationAgentTaskRequestWithItemLocatorDesignatedToIfAgent",
    "AutoEvaluationGuidedDecodingEvaluationTaskConfigurationAutoEvaluationAgentTaskRequestWithItemLocatorDesignatedToIfAgentConfig",
    "AutoEvaluationGuidedDecodingEvaluationTaskConfigurationAutoEvaluationAgentTaskRequestWithItemLocatorDesignatedToTruthfulnessAgent",
    "AutoEvaluationGuidedDecodingEvaluationTaskConfigurationAutoEvaluationAgentTaskRequestWithItemLocatorDesignatedToTruthfulnessAgentConfig",
    "AutoEvaluationGuidedDecodingEvaluationTaskConfigurationAutoEvaluationAgentTaskRequestWithItemLocatorDesignatedToBaseAgent",
    "AutoEvaluationGuidedDecodingEvaluationTaskConfigurationAutoEvaluationAgentTaskRequestWithItemLocatorDesignatedToBaseAgentConfig",
    "AutoEvaluationAgentEvaluationTask",
    "AutoEvaluationAgentEvaluationTaskConfiguration",
    "AutoEvaluationAgentEvaluationTaskConfigurationDesignatedTo",
    "AutoEvaluationAgentEvaluationTaskConfigurationDesignatedToApeAgent",
    "AutoEvaluationAgentEvaluationTaskConfigurationDesignatedToApeAgentConfig",
    "AutoEvaluationAgentEvaluationTaskConfigurationDesignatedToIfAgent",
    "AutoEvaluationAgentEvaluationTaskConfigurationDesignatedToIfAgentConfig",
    "AutoEvaluationAgentEvaluationTaskConfigurationDesignatedToTruthfulnessAgent",
    "AutoEvaluationAgentEvaluationTaskConfigurationDesignatedToTruthfulnessAgentConfig",
    "AutoEvaluationAgentEvaluationTaskConfigurationDesignatedToBaseAgent",
    "AutoEvaluationAgentEvaluationTaskConfigurationDesignatedToBaseAgentConfig",
    "ContributorEvaluationQuestionTask",
    "ContributorEvaluationQuestionTaskConfiguration",
]


class ChatCompletionEvaluationTaskConfigurationTyped(TypedDict, total=False):
    messages: Required[Union[Iterable[Dict[str, object]], ItemLocator]]

    model: Required[str]

    audio: Union[Dict[str, object], ItemLocator]

    frequency_penalty: Union[float, ItemLocator]

    function_call: Union[Dict[str, object], ItemLocator]

    functions: Union[Iterable[Dict[str, object]], ItemLocator]

    logit_bias: Union[Dict[str, int], ItemLocator]

    logprobs: Union[bool, ItemLocator]

    max_completion_tokens: Union[int, ItemLocator]

    max_tokens: Union[int, ItemLocator]

    metadata: Union[Dict[str, str], ItemLocator]

    modalities: Union[SequenceNotStr[str], ItemLocator]

    n: Union[int, ItemLocator]

    parallel_tool_calls: Union[bool, ItemLocator]

    prediction: Union[Dict[str, object], ItemLocator]

    presence_penalty: Union[float, ItemLocator]

    reasoning_effort: str

    response_format: Union[Dict[str, object], ItemLocator]

    seed: Union[int, ItemLocator]

    stop: str

    store: Union[bool, ItemLocator]

    temperature: Union[float, ItemLocator]

    tool_choice: str

    tools: Union[Iterable[Dict[str, object]], ItemLocator]

    top_k: Union[int, ItemLocator]

    top_logprobs: Union[int, ItemLocator]

    top_p: Union[float, ItemLocator]


ChatCompletionEvaluationTaskConfiguration: TypeAlias = Union[
    ChatCompletionEvaluationTaskConfigurationTyped, Dict[str, object]
]


class ChatCompletionEvaluationTask(TypedDict, total=False):
    configuration: Required[ChatCompletionEvaluationTaskConfiguration]

    alias: str
    """Alias to title the results column. Defaults to the `chat_completion`"""

    task_type: Literal["chat_completion"]


class GenericInferenceEvaluationTaskConfigurationInferenceConfigurationLaunchInferenceConfiguration(
    TypedDict, total=False
):
    num_retries: int

    timeout_seconds: int


GenericInferenceEvaluationTaskConfigurationInferenceConfiguration: TypeAlias = Union[
    GenericInferenceEvaluationTaskConfigurationInferenceConfigurationLaunchInferenceConfiguration, ItemLocator
]


class GenericInferenceEvaluationTaskConfiguration(TypedDict, total=False):
    model: Required[str]

    args: Union[Dict[str, object], ItemLocator]

    inference_configuration: GenericInferenceEvaluationTaskConfigurationInferenceConfiguration


class GenericInferenceEvaluationTask(TypedDict, total=False):
    configuration: Required[GenericInferenceEvaluationTaskConfiguration]

    alias: str
    """Alias to title the results column. Defaults to the `inference`"""

    task_type: Literal["inference"]


class ApplicationVariantV1EvaluationTaskConfigurationHistoryApplicationRequestResponsePairArray(TypedDict, total=False):
    request: Required[str]
    """Request inputs"""

    response: Required[str]
    """Response outputs"""

    session_data: Dict[str, object]
    """Session data corresponding to the request response pair"""


class ApplicationVariantV1EvaluationTaskConfigurationOverridesAgenticApplicationOverridesInitialState(
    TypedDict, total=False
):
    current_node: Required[str]

    state: Required[Dict[str, object]]


class ApplicationVariantV1EvaluationTaskConfigurationOverridesAgenticApplicationOverridesPartialTrace(
    TypedDict, total=False
):
    duration_ms: Required[int]

    node_id: Required[str]

    operation_input: Required[str]

    operation_output: Required[str]

    operation_type: Required[str]

    start_timestamp: Required[str]

    workflow_id: Required[str]

    operation_metadata: Dict[str, object]


class ApplicationVariantV1EvaluationTaskConfigurationOverridesAgenticApplicationOverrides(TypedDict, total=False):
    """Execution override options for agentic applications"""

    concurrent: bool

    initial_state: ApplicationVariantV1EvaluationTaskConfigurationOverridesAgenticApplicationOverridesInitialState

    partial_trace: Iterable[
        ApplicationVariantV1EvaluationTaskConfigurationOverridesAgenticApplicationOverridesPartialTrace
    ]

    return_span: bool

    use_channels: bool


ApplicationVariantV1EvaluationTaskConfigurationOverrides: TypeAlias = Union[
    ApplicationVariantV1EvaluationTaskConfigurationOverridesAgenticApplicationOverrides, ItemLocator
]


class ApplicationVariantV1EvaluationTaskConfiguration(TypedDict, total=False):
    application_variant_id: Required[str]

    inputs: Required[Union[Dict[str, object], ItemLocator]]

    history: Union[
        Iterable[ApplicationVariantV1EvaluationTaskConfigurationHistoryApplicationRequestResponsePairArray], ItemLocator
    ]

    operation_metadata: Union[Dict[str, object], ItemLocator]

    overrides: ApplicationVariantV1EvaluationTaskConfigurationOverrides
    """Execution override options for agentic applications"""


class ApplicationVariantV1EvaluationTask(TypedDict, total=False):
    configuration: Required[ApplicationVariantV1EvaluationTaskConfiguration]

    alias: str
    """Alias to title the results column. Defaults to the `application_variant`"""

    task_type: Literal["application_variant"]


class AgentexOutputEvaluationTaskConfiguration(TypedDict, total=False):
    agentex_agent_id: Required[str]

    input_column: Required[Union[ItemLocator, object]]

    include_traces: Union[bool, ItemLocator]


class AgentexOutputEvaluationTask(TypedDict, total=False):
    configuration: Required[AgentexOutputEvaluationTaskConfiguration]

    alias: str
    """Alias to title the results column. Defaults to the `agentex_output`"""

    task_type: Literal["agentex_output"]


class MetricEvaluationTaskConfigurationBleuScorerConfigWithItemLocator(TypedDict, total=False):
    candidate: Required[str]

    reference: Required[str]

    type: Required[Literal["bleu"]]


class MetricEvaluationTaskConfigurationMeteorScorerConfigWithItemLocator(TypedDict, total=False):
    candidate: Required[str]

    reference: Required[str]

    type: Required[Literal["meteor"]]


class MetricEvaluationTaskConfigurationCosineSimilarityScorerConfigWithItemLocator(TypedDict, total=False):
    candidate: Required[str]

    reference: Required[str]

    type: Required[Literal["cosine_similarity"]]


class MetricEvaluationTaskConfigurationF1ScorerConfigWithItemLocator(TypedDict, total=False):
    candidate: Required[str]

    reference: Required[str]

    type: Required[Literal["f1"]]


class MetricEvaluationTaskConfigurationRougeScorer1ConfigWithItemLocator(TypedDict, total=False):
    candidate: Required[str]

    reference: Required[str]

    type: Required[Literal["rouge1"]]


class MetricEvaluationTaskConfigurationRougeScorer2ConfigWithItemLocator(TypedDict, total=False):
    candidate: Required[str]

    reference: Required[str]

    type: Required[Literal["rouge2"]]


class MetricEvaluationTaskConfigurationRougeScorerLConfigWithItemLocator(TypedDict, total=False):
    candidate: Required[str]

    reference: Required[str]

    type: Required[Literal["rougeL"]]


MetricEvaluationTaskConfiguration: TypeAlias = Union[
    MetricEvaluationTaskConfigurationBleuScorerConfigWithItemLocator,
    MetricEvaluationTaskConfigurationMeteorScorerConfigWithItemLocator,
    MetricEvaluationTaskConfigurationCosineSimilarityScorerConfigWithItemLocator,
    MetricEvaluationTaskConfigurationF1ScorerConfigWithItemLocator,
    MetricEvaluationTaskConfigurationRougeScorer1ConfigWithItemLocator,
    MetricEvaluationTaskConfigurationRougeScorer2ConfigWithItemLocator,
    MetricEvaluationTaskConfigurationRougeScorerLConfigWithItemLocator,
]


class MetricEvaluationTask(TypedDict, total=False):
    configuration: Required[MetricEvaluationTaskConfiguration]

    alias: str
    """Alias to title the results column.

    Defaults to the metric type specified in the configuration
    """

    task_type: Literal["metric"]


class AutoEvaluationQuestionTaskConfiguration(TypedDict, total=False):
    model: Required[str]
    """model specified as `model_vendor/model_name`"""

    prompt: Required[str]

    question_id: Required[str]
    """question to be evaluated"""


class AutoEvaluationQuestionTask(TypedDict, total=False):
    configuration: Required[AutoEvaluationQuestionTaskConfiguration]

    alias: str
    """Alias to title the results column. Defaults to the `auto_evaluation_question`"""

    task_type: Literal["auto_evaluation.question"]


class AutoEvaluationGuidedDecodingEvaluationTaskConfigurationAutoEvaluationStructuredOutputTaskRequestWithItemLocator(
    TypedDict, total=False
):
    model: Required[str]
    """model specified as `model_vendor/model_name`"""

    prompt: Required[str]

    response_format: Required[Dict[str, object]]
    """JSON schema used for structuring the model response"""

    inference_args: Dict[str, object]
    """Additional arguments to pass to the inference request"""

    system_prompt: str


class AutoEvaluationGuidedDecodingEvaluationTaskConfigurationAutoEvaluationGuidedDecodingTaskRequestWithItemLocator(
    TypedDict, total=False
):
    choices: Required[SequenceNotStr[str]]
    """Choices array cannot be empty"""

    model: Required[str]
    """model specified as `model_vendor/model_name`"""

    prompt: Required[str]

    inference_args: Dict[str, object]
    """Additional arguments to pass to the inference request"""

    system_prompt: str


class AutoEvaluationGuidedDecodingEvaluationTaskConfigurationAutoEvaluationAgentTaskRequestWithItemLocatorDesignatedToApeAgentConfig(
    TypedDict, total=False
):
    model: str

    temperature: float


class AutoEvaluationGuidedDecodingEvaluationTaskConfigurationAutoEvaluationAgentTaskRequestWithItemLocatorDesignatedToApeAgent(
    TypedDict, total=False
):
    config: Required[
        AutoEvaluationGuidedDecodingEvaluationTaskConfigurationAutoEvaluationAgentTaskRequestWithItemLocatorDesignatedToApeAgentConfig
    ]

    agent_name: Literal["APEAgent"]


class AutoEvaluationGuidedDecodingEvaluationTaskConfigurationAutoEvaluationAgentTaskRequestWithItemLocatorDesignatedToIfAgentConfig(
    TypedDict, total=False
):
    model: str


class AutoEvaluationGuidedDecodingEvaluationTaskConfigurationAutoEvaluationAgentTaskRequestWithItemLocatorDesignatedToIfAgent(
    TypedDict, total=False
):
    config: Required[
        AutoEvaluationGuidedDecodingEvaluationTaskConfigurationAutoEvaluationAgentTaskRequestWithItemLocatorDesignatedToIfAgentConfig
    ]

    agent_name: Literal["IFAgent"]


class AutoEvaluationGuidedDecodingEvaluationTaskConfigurationAutoEvaluationAgentTaskRequestWithItemLocatorDesignatedToTruthfulnessAgentConfig(
    TypedDict, total=False
):
    model: str


class AutoEvaluationGuidedDecodingEvaluationTaskConfigurationAutoEvaluationAgentTaskRequestWithItemLocatorDesignatedToTruthfulnessAgent(
    TypedDict, total=False
):
    config: Required[
        AutoEvaluationGuidedDecodingEvaluationTaskConfigurationAutoEvaluationAgentTaskRequestWithItemLocatorDesignatedToTruthfulnessAgentConfig
    ]

    agent_name: Literal["TruthfulnessAgent"]


class AutoEvaluationGuidedDecodingEvaluationTaskConfigurationAutoEvaluationAgentTaskRequestWithItemLocatorDesignatedToBaseAgentConfig(
    TypedDict, total=False
):
    model: str


class AutoEvaluationGuidedDecodingEvaluationTaskConfigurationAutoEvaluationAgentTaskRequestWithItemLocatorDesignatedToBaseAgent(
    TypedDict, total=False
):
    config: Required[
        AutoEvaluationGuidedDecodingEvaluationTaskConfigurationAutoEvaluationAgentTaskRequestWithItemLocatorDesignatedToBaseAgentConfig
    ]

    agent_name: Literal["BaseAgent"]


AutoEvaluationGuidedDecodingEvaluationTaskConfigurationAutoEvaluationAgentTaskRequestWithItemLocatorDesignatedTo: TypeAlias = Union[
    AutoEvaluationGuidedDecodingEvaluationTaskConfigurationAutoEvaluationAgentTaskRequestWithItemLocatorDesignatedToApeAgent,
    AutoEvaluationGuidedDecodingEvaluationTaskConfigurationAutoEvaluationAgentTaskRequestWithItemLocatorDesignatedToIfAgent,
    AutoEvaluationGuidedDecodingEvaluationTaskConfigurationAutoEvaluationAgentTaskRequestWithItemLocatorDesignatedToTruthfulnessAgent,
    AutoEvaluationGuidedDecodingEvaluationTaskConfigurationAutoEvaluationAgentTaskRequestWithItemLocatorDesignatedToBaseAgent,
]


class AutoEvaluationGuidedDecodingEvaluationTaskConfigurationAutoEvaluationAgentTaskRequestWithItemLocator(
    TypedDict, total=False
):
    definition: Required[str]

    name: Required[str]

    output_rules: Required[SequenceNotStr[str]]

    data_fields: SequenceNotStr[str]

    designated_to: (
        AutoEvaluationGuidedDecodingEvaluationTaskConfigurationAutoEvaluationAgentTaskRequestWithItemLocatorDesignatedTo
    )

    output_type: Literal["text", "integer", "float", "boolean"]

    output_values: SequenceNotStr[Union[str, float, bool]]


AutoEvaluationGuidedDecodingEvaluationTaskConfiguration: TypeAlias = Union[
    AutoEvaluationGuidedDecodingEvaluationTaskConfigurationAutoEvaluationStructuredOutputTaskRequestWithItemLocator,
    AutoEvaluationGuidedDecodingEvaluationTaskConfigurationAutoEvaluationGuidedDecodingTaskRequestWithItemLocator,
    AutoEvaluationGuidedDecodingEvaluationTaskConfigurationAutoEvaluationAgentTaskRequestWithItemLocator,
]


class AutoEvaluationGuidedDecodingEvaluationTask(TypedDict, total=False):
    configuration: Required[AutoEvaluationGuidedDecodingEvaluationTaskConfiguration]

    alias: str
    """Alias to title the results column.

    Defaults to the `auto_evaluation_guided_decoding`
    """

    task_type: Literal["auto_evaluation.guided_decoding"]


class AutoEvaluationAgentEvaluationTaskConfigurationDesignatedToApeAgentConfig(TypedDict, total=False):
    model: str

    temperature: float


class AutoEvaluationAgentEvaluationTaskConfigurationDesignatedToApeAgent(TypedDict, total=False):
    config: Required[AutoEvaluationAgentEvaluationTaskConfigurationDesignatedToApeAgentConfig]

    agent_name: Literal["APEAgent"]


class AutoEvaluationAgentEvaluationTaskConfigurationDesignatedToIfAgentConfig(TypedDict, total=False):
    model: str


class AutoEvaluationAgentEvaluationTaskConfigurationDesignatedToIfAgent(TypedDict, total=False):
    config: Required[AutoEvaluationAgentEvaluationTaskConfigurationDesignatedToIfAgentConfig]

    agent_name: Literal["IFAgent"]


class AutoEvaluationAgentEvaluationTaskConfigurationDesignatedToTruthfulnessAgentConfig(TypedDict, total=False):
    model: str


class AutoEvaluationAgentEvaluationTaskConfigurationDesignatedToTruthfulnessAgent(TypedDict, total=False):
    config: Required[AutoEvaluationAgentEvaluationTaskConfigurationDesignatedToTruthfulnessAgentConfig]

    agent_name: Literal["TruthfulnessAgent"]


class AutoEvaluationAgentEvaluationTaskConfigurationDesignatedToBaseAgentConfig(TypedDict, total=False):
    model: str


class AutoEvaluationAgentEvaluationTaskConfigurationDesignatedToBaseAgent(TypedDict, total=False):
    config: Required[AutoEvaluationAgentEvaluationTaskConfigurationDesignatedToBaseAgentConfig]

    agent_name: Literal["BaseAgent"]


AutoEvaluationAgentEvaluationTaskConfigurationDesignatedTo: TypeAlias = Union[
    AutoEvaluationAgentEvaluationTaskConfigurationDesignatedToApeAgent,
    AutoEvaluationAgentEvaluationTaskConfigurationDesignatedToIfAgent,
    AutoEvaluationAgentEvaluationTaskConfigurationDesignatedToTruthfulnessAgent,
    AutoEvaluationAgentEvaluationTaskConfigurationDesignatedToBaseAgent,
]


class AutoEvaluationAgentEvaluationTaskConfiguration(TypedDict, total=False):
    definition: Required[str]

    name: Required[str]

    output_rules: Required[SequenceNotStr[str]]

    data_fields: SequenceNotStr[str]

    designated_to: AutoEvaluationAgentEvaluationTaskConfigurationDesignatedTo

    output_type: Literal["text", "integer", "float", "boolean"]

    output_values: SequenceNotStr[Union[str, float, bool]]


class AutoEvaluationAgentEvaluationTask(TypedDict, total=False):
    configuration: Required[AutoEvaluationAgentEvaluationTaskConfiguration]

    alias: str
    """Alias to title the results column. Defaults to the `auto_evaluation_agent`"""

    task_type: Literal["auto_evaluation.agent"]


class ContributorEvaluationQuestionTaskConfiguration(TypedDict, total=False):
    layout: Required["ContainerParam"]

    question_id: Required[str]

    queue_id: str
    """The contributor annotation queue to include this task in. Defaults to `default`"""

    required: bool
    """Whether the question is required to be answered"""


class ContributorEvaluationQuestionTask(TypedDict, total=False):
    configuration: Required[ContributorEvaluationQuestionTaskConfiguration]

    alias: str
    """Alias to title the results column.

    Defaults to the `contributor_evaluation_question`
    """

    task_type: Literal["contributor_evaluation.question"]


EvaluationTaskParam: TypeAlias = Union[
    ChatCompletionEvaluationTask,
    GenericInferenceEvaluationTask,
    ApplicationVariantV1EvaluationTask,
    AgentexOutputEvaluationTask,
    MetricEvaluationTask,
    AutoEvaluationQuestionTask,
    AutoEvaluationGuidedDecodingEvaluationTask,
    AutoEvaluationAgentEvaluationTask,
    ContributorEvaluationQuestionTask,
]

from .container_param import ContainerParam



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["DatasetItemRetrieveParams"]


class DatasetItemRetrieveParams(TypedDict, total=False):
    version: int
    """Optional dataset version. When unset, returns the latest version."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from scale_gp_beta import SGPClient, AsyncSGPClient
from scale_gp_beta.types import (
    VectorStore,
    VectorStoreDropResponse,
    VectorStoreCountResponse,
    VectorStoreQueryResponse,
    VectorStoreDeleteResponse,
    VectorStoreUpsertResponse,
    VectorStoreVectorsResponse,
)
from scale_gp_beta.pagination import SyncCursorPage, AsyncCursorPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestVectorStores:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: SGPClient) -> None:
        vector_store = client.vector_stores.create(
            embedding_model="sentence-transformers/all-MiniLM-L12-v2",
            name="name",
        )
        assert_matches_type(VectorStore, vector_store, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: SGPClient) -> None:
        vector_store = client.vector_stores.create(
            embedding_model="sentence-transformers/all-MiniLM-L12-v2",
            name="name",
            indexed_metadata_fields={"foo": "string"},
        )
        assert_matches_type(VectorStore, vector_store, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: SGPClient) -> None:
        response = client.vector_stores.with_raw_response.create(
            embedding_model="sentence-transformers/all-MiniLM-L12-v2",
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        vector_store = response.parse()
        assert_matches_type(VectorStore, vector_store, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: SGPClient) -> None:
        with client.vector_stores.with_streaming_response.create(
            embedding_model="sentence-transformers/all-MiniLM-L12-v2",
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            vector_store = response.parse()
            assert_matches_type(VectorStore, vector_store, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: SGPClient) -> None:
        vector_store = client.vector_stores.retrieve(
            "vector_store_name",
        )
        assert_matches_type(VectorStore, vector_store, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: SGPClient) -> None:
        response = client.vector_stores.with_raw_response.retrieve(
            "vector_store_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        vector_store = response.parse()
        assert_matches_type(VectorStore, vector_store, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: SGPClient) -> None:
        with client.vector_stores.with_streaming_response.retrieve(
            "vector_store_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            vector_store = response.parse()
            assert_matches_type(VectorStore, vector_store, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `vector_store_name` but received ''"):
            client.vector_stores.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_list(self, client: SGPClient) -> None:
        vector_store = client.vector_stores.list()
        assert_matches_type(SyncCursorPage[VectorStore], vector_store, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: SGPClient) -> None:
        vector_store = client.vector_stores.list(
            ending_before="ending_before",
            limit=1,
            sort_by="sort_by",
            sort_order="asc",
            starting_after="starting_after",
        )
        assert_matches_type(SyncCursorPage[VectorStore], vector_store, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: SGPClient) -> None:
        response = client.vector_stores.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        vector_store = response.parse()
        assert_matches_type(SyncCursorPage[VectorStore], vector_store, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: SGPClient) -> None:
        with client.vector_stores.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            vector_store = response.parse()
            assert_matches_type(SyncCursorPage[VectorStore], vector_store, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: SGPClient) -> None:
        vector_store = client.vector_stores.delete(
            vector_store_name="vector_store_name",
        )
        assert_matches_type(VectorStoreDeleteResponse, vector_store, path=["response"])

    @parametrize
    def test_method_delete_with_all_params(self, client: SGPClient) -> None:
        vector_store = client.vector_stores.delete(
            vector_store_name="vector_store_name",
            filter={"foo": "bar"},
            ids=["string"],
        )
        assert_matches_type(VectorStoreDeleteResponse, vector_store, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: SGPClient) -> None:
        response = client.vector_stores.with_raw_response.delete(
            vector_store_name="vector_store_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        vector_store = response.parse()
        assert_matches_type(VectorStoreDeleteResponse, vector_store, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: SGPClient) -> None:
        with client.vector_stores.with_streaming_response.delete(
            vector_store_name="vector_store_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            vector_store = response.parse()
            assert_matches_type(VectorStoreDeleteResponse, vector_store, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `vector_store_name` but received ''"):
            client.vector_stores.with_raw_response.delete(
                vector_store_name="",
            )

    @parametrize
    def test_method_configure(self, client: SGPClient) -> None:
        vector_store = client.vector_stores.configure(
            vector_store_name="vector_store_name",
            indexed_metadata_fields={"foo": "string"},
        )
        assert_matches_type(VectorStore, vector_store, path=["response"])

    @parametrize
    def test_raw_response_configure(self, client: SGPClient) -> None:
        response = client.vector_stores.with_raw_response.configure(
            vector_store_name="vector_store_name",
            indexed_metadata_fields={"foo": "string"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        vector_store = response.parse()
        assert_matches_type(VectorStore, vector_store, path=["response"])

    @parametrize
    def test_streaming_response_configure(self, client: SGPClient) -> None:
        with client.vector_stores.with_streaming_response.configure(
            vector_store_name="vector_store_name",
            indexed_metadata_fields={"foo": "string"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            vector_store = response.parse()
            assert_matches_type(VectorStore, vector_store, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_configure(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `vector_store_name` but received ''"):
            client.vector_stores.with_raw_response.configure(
                vector_store_name="",
                indexed_metadata_fields={"foo": "string"},
            )

    @parametrize
    def test_method_count(self, client: SGPClient) -> None:
        vector_store = client.vector_stores.count(
            vector_store_name="vector_store_name",
        )
        assert_matches_type(VectorStoreCountResponse, vector_store, path=["response"])

    @parametrize
    def test_method_count_with_all_params(self, client: SGPClient) -> None:
        vector_store = client.vector_stores.count(
            vector_store_name="vector_store_name",
            filter={"foo": "bar"},
        )
        assert_matches_type(VectorStoreCountResponse, vector_store, path=["response"])

    @parametrize
    def test_raw_response_count(self, client: SGPClient) -> None:
        response = client.vector_stores.with_raw_response.count(
            vector_store_name="vector_store_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        vector_store = response.parse()
        assert_matches_type(VectorStoreCountResponse, vector_store, path=["response"])

    @parametrize
    def test_streaming_response_count(self, client: SGPClient) -> None:
        with client.vector_stores.with_streaming_response.count(
            vector_store_name="vector_store_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            vector_store = response.parse()
            assert_matches_type(VectorStoreCountResponse, vector_store, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_count(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `vector_store_name` but received ''"):
            client.vector_stores.with_raw_response.count(
                vector_store_name="",
            )

    @parametrize
    def test_method_drop(self, client: SGPClient) -> None:
        vector_store = client.vector_stores.drop(
            "vector_store_name",
        )
        assert_matches_type(VectorStoreDropResponse, vector_store, path=["response"])

    @parametrize
    def test_raw_response_drop(self, client: SGPClient) -> None:
        response = client.vector_stores.with_raw_response.drop(
            "vector_store_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        vector_store = response.parse()
        assert_matches_type(VectorStoreDropResponse, vector_store, path=["response"])

    @parametrize
    def test_streaming_response_drop(self, client: SGPClient) -> None:
        with client.vector_stores.with_streaming_response.drop(
            "vector_store_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            vector_store = response.parse()
            assert_matches_type(VectorStoreDropResponse, vector_store, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_drop(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `vector_store_name` but received ''"):
            client.vector_stores.with_raw_response.drop(
                "",
            )

    @parametrize
    def test_method_query(self, client: SGPClient) -> None:
        vector_store = client.vector_stores.query(
            vector_store_name="vector_store_name",
            text="text",
        )
        assert_matches_type(VectorStoreQueryResponse, vector_store, path=["response"])

    @parametrize
    def test_method_query_with_all_params(self, client: SGPClient) -> None:
        vector_store = client.vector_stores.query(
            vector_store_name="vector_store_name",
            text="text",
            filter={"foo": "bar"},
            include_vectors=True,
            query_type="semantic",
            rerank=True,
            rerank_model="rerank_model",
            rerank_top_n=0,
            top_k=1,
        )
        assert_matches_type(VectorStoreQueryResponse, vector_store, path=["response"])

    @parametrize
    def test_raw_response_query(self, client: SGPClient) -> None:
        response = client.vector_stores.with_raw_response.query(
            vector_store_name="vector_store_name",
            text="text",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        vector_store = response.parse()
        assert_matches_type(VectorStoreQueryResponse, vector_store, path=["response"])

    @parametrize
    def test_streaming_response_query(self, client: SGPClient) -> None:
        with client.vector_stores.with_streaming_response.query(
            vector_store_name="vector_store_name",
            text="text",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            vector_store = response.parse()
            assert_matches_type(VectorStoreQueryResponse, vector_store, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_query(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `vector_store_name` but received ''"):
            client.vector_stores.with_raw_response.query(
                vector_store_name="",
                text="text",
            )

    @parametrize
    def test_method_upsert(self, client: SGPClient) -> None:
        vector_store = client.vector_stores.upsert(
            vector_store_name="vector_store_name",
            vectors=[{}],
        )
        assert_matches_type(VectorStoreUpsertResponse, vector_store, path=["response"])

    @parametrize
    def test_raw_response_upsert(self, client: SGPClient) -> None:
        response = client.vector_stores.with_raw_response.upsert(
            vector_store_name="vector_store_name",
            vectors=[{}],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        vector_store = response.parse()
        assert_matches_type(VectorStoreUpsertResponse, vector_store, path=["response"])

    @parametrize
    def test_streaming_response_upsert(self, client: SGPClient) -> None:
        with client.vector_stores.with_streaming_response.upsert(
            vector_store_name="vector_store_name",
            vectors=[{}],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            vector_store = response.parse()
            assert_matches_type(VectorStoreUpsertResponse, vector_store, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_upsert(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `vector_store_name` but received ''"):
            client.vector_stores.with_raw_response.upsert(
                vector_store_name="",
                vectors=[{}],
            )

    @parametrize
    def test_method_vectors(self, client: SGPClient) -> None:
        vector_store = client.vector_stores.vectors(
            vector_store_name="vector_store_name",
        )
        assert_matches_type(VectorStoreVectorsResponse, vector_store, path=["response"])

    @parametrize
    def test_method_vectors_with_all_params(self, client: SGPClient) -> None:
        vector_store = client.vector_stores.vectors(
            vector_store_name="vector_store_name",
            cursor="cursor",
            filter="filter",
            include_vectors=True,
            limit=1,
        )
        assert_matches_type(VectorStoreVectorsResponse, vector_store, path=["response"])

    @parametrize
    def test_raw_response_vectors(self, client: SGPClient) -> None:
        response = client.vector_stores.with_raw_response.vectors(
            vector_store_name="vector_store_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        vector_store = response.parse()
        assert_matches_type(VectorStoreVectorsResponse, vector_store, path=["response"])

    @parametrize
    def test_streaming_response_vectors(self, client: SGPClient) -> None:
        with client.vector_stores.with_streaming_response.vectors(
            vector_store_name="vector_store_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            vector_store = response.parse()
            assert_matches_type(VectorStoreVectorsResponse, vector_store, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_vectors(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `vector_store_name` but received ''"):
            client.vector_stores.with_raw_response.vectors(
                vector_store_name="",
            )


class TestAsyncVectorStores:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncSGPClient) -> None:
        vector_store = await async_client.vector_stores.create(
            embedding_model="sentence-transformers/all-MiniLM-L12-v2",
            name="name",
        )
        assert_matches_type(VectorStore, vector_store, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncSGPClient) -> None:
        vector_store = await async_client.vector_stores.create(
            embedding_model="sentence-transformers/all-MiniLM-L12-v2",
            name="name",
            indexed_metadata_fields={"foo": "string"},
        )
        assert_matches_type(VectorStore, vector_store, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.vector_stores.with_raw_response.create(
            embedding_model="sentence-transformers/all-MiniLM-L12-v2",
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        vector_store = await response.parse()
        assert_matches_type(VectorStore, vector_store, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncSGPClient) -> None:
        async with async_client.vector_stores.with_streaming_response.create(
            embedding_model="sentence-transformers/all-MiniLM-L12-v2",
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            vector_store = await response.parse()
            assert_matches_type(VectorStore, vector_store, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncSGPClient) -> None:
        vector_store = await async_client.vector_stores.retrieve(
            "vector_store_name",
        )
        assert_matches_type(VectorStore, vector_store, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.vector_stores.with_raw_response.retrieve(
            "vector_store_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        vector_store = await response.parse()
        assert_matches_type(VectorStore, vector_store, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        async with async_client.vector_stores.with_streaming_response.retrieve(
            "vector_store_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            vector_store = await response.parse()
            assert_matches_type(VectorStore, vector_store, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `vector_store_name` but received ''"):
            await async_client.vector_stores.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncSGPClient) -> None:
        vector_store = await async_client.vector_stores.list()
        assert_matches_type(AsyncCursorPage[VectorStore], vector_store, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncSGPClient) -> None:
        vector_store = await async_client.vector_stores.list(
            ending_before="ending_before",
            limit=1,
            sort_by="sort_by",
            sort_order="asc",
            starting_after="starting_after",
        )
        assert_matches_type(AsyncCursorPage[VectorStore], vector_store, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.vector_stores.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        vector_store = await response.parse()
        assert_matches_type(AsyncCursorPage[VectorStore], vector_store, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncSGPClient) -> None:
        async with async_client.vector_stores.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            vector_store = await response.parse()
            assert_matches_type(AsyncCursorPage[VectorStore], vector_store, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncSGPClient) -> None:
        vector_store = await async_client.vector_stores.delete(
            vector_store_name="vector_store_name",
        )
        assert_matches_type(VectorStoreDeleteResponse, vector_store, path=["response"])

    @parametrize
    async def test_method_delete_with_all_params(self, async_client: AsyncSGPClient) -> None:
        vector_store = await async_client.vector_stores.delete(
            vector_store_name="vector_store_name",
            filter={"foo": "bar"},
            ids=["string"],
        )
        assert_matches_type(VectorStoreDeleteResponse, vector_store, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.vector_stores.with_raw_response.delete(
            vector_store_name="vector_store_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        vector_store = await response.parse()
        assert_matches_type(VectorStoreDeleteResponse, vector_store, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncSGPClient) -> None:
        async with async_client.vector_stores.with_streaming_response.delete(
            vector_store_name="vector_store_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            vector_store = await response.parse()
            assert_matches_type(VectorStoreDeleteResponse, vector_store, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `vector_store_name` but received ''"):
            await async_client.vector_stores.with_raw_response.delete(
                vector_store_name="",
            )

    @parametrize
    async def test_method_configure(self, async_client: AsyncSGPClient) -> None:
        vector_store = await async_client.vector_stores.configure(
            vector_store_name="vector_store_name",
            indexed_metadata_fields={"foo": "string"},
        )
        assert_matches_type(VectorStore, vector_store, path=["response"])

    @parametrize
    async def test_raw_response_configure(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.vector_stores.with_raw_response.configure(
            vector_store_name="vector_store_name",
            indexed_metadata_fields={"foo": "string"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        vector_store = await response.parse()
        assert_matches_type(VectorStore, vector_store, path=["response"])

    @parametrize
    async def test_streaming_response_configure(self, async_client: AsyncSGPClient) -> None:
        async with async_client.vector_stores.with_streaming_response.configure(
            vector_store_name="vector_store_name",
            indexed_metadata_fields={"foo": "string"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            vector_store = await response.parse()
            assert_matches_type(VectorStore, vector_store, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_configure(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `vector_store_name` but received ''"):
            await async_client.vector_stores.with_raw_response.configure(
                vector_store_name="",
                indexed_metadata_fields={"foo": "string"},
            )

    @parametrize
    async def test_method_count(self, async_client: AsyncSGPClient) -> None:
        vector_store = await async_client.vector_stores.count(
            vector_store_name="vector_store_name",
        )
        assert_matches_type(VectorStoreCountResponse, vector_store, path=["response"])

    @parametrize
    async def test_method_count_with_all_params(self, async_client: AsyncSGPClient) -> None:
        vector_store = await async_client.vector_stores.count(
            vector_store_name="vector_store_name",
            filter={"foo": "bar"},
        )
        assert_matches_type(VectorStoreCountResponse, vector_store, path=["response"])

    @parametrize
    async def test_raw_response_count(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.vector_stores.with_raw_response.count(
            vector_store_name="vector_store_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        vector_store = await response.parse()
        assert_matches_type(VectorStoreCountResponse, vector_store, path=["response"])

    @parametrize
    async def test_streaming_response_count(self, async_client: AsyncSGPClient) -> None:
        async with async_client.vector_stores.with_streaming_response.count(
            vector_store_name="vector_store_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            vector_store = await response.parse()
            assert_matches_type(VectorStoreCountResponse, vector_store, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_count(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `vector_store_name` but received ''"):
            await async_client.vector_stores.with_raw_response.count(
                vector_store_name="",
            )

    @parametrize
    async def test_method_drop(self, async_client: AsyncSGPClient) -> None:
        vector_store = await async_client.vector_stores.drop(
            "vector_store_name",
        )
        assert_matches_type(VectorStoreDropResponse, vector_store, path=["response"])

    @parametrize
    async def test_raw_response_drop(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.vector_stores.with_raw_response.drop(
            "vector_store_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        vector_store = await response.parse()
        assert_matches_type(VectorStoreDropResponse, vector_store, path=["response"])

    @parametrize
    async def test_streaming_response_drop(self, async_client: AsyncSGPClient) -> None:
        async with async_client.vector_stores.with_streaming_response.drop(
            "vector_store_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            vector_store = await response.parse()
            assert_matches_type(VectorStoreDropResponse, vector_store, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_drop(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `vector_store_name` but received ''"):
            await async_client.vector_stores.with_raw_response.drop(
                "",
            )

    @parametrize
    async def test_method_query(self, async_client: AsyncSGPClient) -> None:
        vector_store = await async_client.vector_stores.query(
            vector_store_name="vector_store_name",
            text="text",
        )
        assert_matches_type(VectorStoreQueryResponse, vector_store, path=["response"])

    @parametrize
    async def test_method_query_with_all_params(self, async_client: AsyncSGPClient) -> None:
        vector_store = await async_client.vector_stores.query(
            vector_store_name="vector_store_name",
            text="text",
            filter={"foo": "bar"},
            include_vectors=True,
            query_type="semantic",
            rerank=True,
            rerank_model="rerank_model",
            rerank_top_n=0,
            top_k=1,
        )
        assert_matches_type(VectorStoreQueryResponse, vector_store, path=["response"])

    @parametrize
    async def test_raw_response_query(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.vector_stores.with_raw_response.query(
            vector_store_name="vector_store_name",
            text="text",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        vector_store = await response.parse()
        assert_matches_type(VectorStoreQueryResponse, vector_store, path=["response"])

    @parametrize
    async def test_streaming_response_query(self, async_client: AsyncSGPClient) -> None:
        async with async_client.vector_stores.with_streaming_response.query(
            vector_store_name="vector_store_name",
            text="text",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            vector_store = await response.parse()
            assert_matches_type(VectorStoreQueryResponse, vector_store, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_query(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `vector_store_name` but received ''"):
            await async_client.vector_stores.with_raw_response.query(
                vector_store_name="",
                text="text",
            )

    @parametrize
    async def test_method_upsert(self, async_client: AsyncSGPClient) -> None:
        vector_store = await async_client.vector_stores.upsert(
            vector_store_name="vector_store_name",
            vectors=[{}],
        )
        assert_matches_type(VectorStoreUpsertResponse, vector_store, path=["response"])

    @parametrize
    async def test_raw_response_upsert(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.vector_stores.with_raw_response.upsert(
            vector_store_name="vector_store_name",
            vectors=[{}],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        vector_store = await response.parse()
        assert_matches_type(VectorStoreUpsertResponse, vector_store, path=["response"])

    @parametrize
    async def test_streaming_response_upsert(self, async_client: AsyncSGPClient) -> None:
        async with async_client.vector_stores.with_streaming_response.upsert(
            vector_store_name="vector_store_name",
            vectors=[{}],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            vector_store = await response.parse()
            assert_matches_type(VectorStoreUpsertResponse, vector_store, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_upsert(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `vector_store_name` but received ''"):
            await async_client.vector_stores.with_raw_response.upsert(
                vector_store_name="",
                vectors=[{}],
            )

    @parametrize
    async def test_method_vectors(self, async_client: AsyncSGPClient) -> None:
        vector_store = await async_client.vector_stores.vectors(
            vector_store_name="vector_store_name",
        )
        assert_matches_type(VectorStoreVectorsResponse, vector_store, path=["response"])

    @parametrize
    async def test_method_vectors_with_all_params(self, async_client: AsyncSGPClient) -> None:
        vector_store = await async_client.vector_stores.vectors(
            vector_store_name="vector_store_name",
            cursor="cursor",
            filter="filter",
            include_vectors=True,
            limit=1,
        )
        assert_matches_type(VectorStoreVectorsResponse, vector_store, path=["response"])

    @parametrize
    async def test_raw_response_vectors(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.vector_stores.with_raw_response.vectors(
            vector_store_name="vector_store_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        vector_store = await response.parse()
        assert_matches_type(VectorStoreVectorsResponse, vector_store, path=["response"])

    @parametrize
    async def test_streaming_response_vectors(self, async_client: AsyncSGPClient) -> None:
        async with async_client.vector_stores.with_streaming_response.vectors(
            vector_store_name="vector_store_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            vector_store = await response.parse()
            assert_matches_type(VectorStoreVectorsResponse, vector_store, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_vectors(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `vector_store_name` but received ''"):
            await async_client.vector_stores.with_raw_response.vectors(
                vector_store_name="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable
from typing_extensions import Required, TypedDict

__all__ = ["DatasetItemBatchCreateParams"]


class DatasetItemBatchCreateParams(TypedDict, total=False):
    data: Required[Iterable[Dict[str, object]]]
    """Items to be added to the dataset"""

    dataset_id: Required[str]
    """Identifier of the target dataset"""

    files: Iterable[Dict[str, str]]
    """Files to be associated to the dataset"""



import logging
import contextvars
from types import TracebackType
from typing import Type, Optional
from typing_extensions import override

from scale_gp_beta.types import SpanType, SpanStatus

from .span import Span, BaseSpan, NoOpSpan
from .util import generate_trace_id
from .scope import Scope
from .types import SpanInputParam, SpanOutputParam, SpanMetadataParam
from .trace_queue_manager import TraceQueueManager

log: logging.Logger = logging.getLogger(__name__)


class BaseTrace:
    def __init__(
            self,
            queue_manager: Optional[TraceQueueManager],
            root_span: BaseSpan,
            trace_id: str
    ) -> None:
        self._trace_id = trace_id
        self.queue_manager = queue_manager

        self._in_progress = False
        self._contextvar_token: Optional[contextvars.Token[Optional[BaseTrace]]] = None

        self.root_span = root_span

    def start(self) -> None:
        pass

    def end(self) -> None:
        pass

    def flush(self, blocking: bool = True) -> None:
        pass

    @property
    def metadata(self) -> SpanMetadataParam:
        return self.root_span.metadata

    @metadata.setter
    def metadata(self, value: SpanMetadataParam) -> None:
        self.root_span.metadata = value

    @property
    def input(self) -> SpanInputParam:
        return self.root_span.input

    @input.setter
    def input(self, value: SpanInputParam) -> None:
        self.root_span.input = value

    @property
    def output(self) -> SpanOutputParam:
        return self.root_span.output

    @output.setter
    def output(self, value: SpanOutputParam) -> None:
        self.root_span.output = value

    # no setters
    @property
    def name(self) -> Optional[str]:
        return self.root_span.name

    @property
    def span_id(self) -> Optional[str]:
        return self.root_span.span_id

    @property
    def trace_id(self) -> Optional[str]:
        return self._trace_id

    @property
    def group_id(self) -> Optional[str]:
        return self.root_span.group_id

    @property
    def span_type(self) -> SpanType:
        return self.root_span.span_type

    @property
    def status(self) -> SpanStatus:
        return self.root_span.status

    def set_error(
            self,
            error_type: Optional[str] = None,
            error_message: Optional[str] = None,
            exception: Optional[BaseException] = None,
    ) -> None:
        self.root_span.set_error(error_type=error_type, error_message=error_message, exception=exception)

    def __enter__(self) -> "BaseTrace":
        self.start()
        return self

    def __exit__(
        self,
        exc_type: Optional[Type[BaseException]] = None,
        exc_value: Optional[BaseException] = None,
        traceback: Optional[TracebackType] = None,
    ) -> None:
        self.end()

    @override
    def __repr__(self) -> str:
        return (
            f"{self.__class__.__name__}("
            f"trace_id='{self.trace_id}', "
            f"root_span='{repr(self.root_span)}', "
            ")"
        )

    @override
    def __str__(self) -> str:
        return self.__repr__()


class NoOpTrace(BaseTrace):
    def __init__(
            self,
            name: str,
            queue_manager: Optional[TraceQueueManager] = None,
            trace_id: Optional[str] = None,
            span_id: Optional[str] = None,
            group_id: Optional[str] = None,
            span_type: SpanType = "TRACER",
            input: Optional[SpanInputParam] = None,
            output: Optional[SpanOutputParam] = None,
            metadata: Optional[SpanMetadataParam] = None,
    ):
        trace_id = trace_id or generate_trace_id()
        root_span = NoOpSpan(
            name=name,
            span_id=span_id,
            trace_id=trace_id,
            group_id=group_id,
            queue_manager=queue_manager,
            metadata=metadata,
            span_type=span_type,
            input=input,
            output=output,
        )
        super().__init__(queue_manager, root_span, trace_id)

    @override
    def start(self) -> None:
        self.root_span.start()

    @override
    def end(self) -> None:
        self.root_span.end()


class Trace(BaseTrace):
    def __init__(
            self,
            name: str,
            queue_manager: TraceQueueManager,
            trace_id: Optional[str] = None,
            span_id: Optional[str] = None,
            group_id: Optional[str] = None,
            span_type: SpanType = "TRACER",
            input: Optional[SpanInputParam] = None,
            output: Optional[SpanOutputParam] = None,
            metadata: Optional[SpanMetadataParam] = None,
    ):
        trace_id = trace_id or generate_trace_id()
        root_span = Span(
            name=name,
            span_id=span_id,
            trace_id=trace_id,
            group_id=group_id,
            queue_manager=queue_manager,
            metadata=metadata,
            span_type=span_type,
            input=input,
            output=output,
        )
        super().__init__(queue_manager, root_span, trace_id)
        self.queue_manager: TraceQueueManager = queue_manager

    @override
    def start(self) -> None:
        if self._in_progress:
            log.warning(f"Trace already started: {self.trace_id}")
            return

        self._in_progress = True
        self.queue_manager.report_trace_start(self)  # no-op
        self._contextvar_token = Scope.set_current_trace(self)

        self.root_span.start()

    @override
    def end(self) -> None:
        if not self._in_progress:
            log.warning(f"Ending trace which is not active: {self.trace_id}")
            return
        if self._contextvar_token is None:
            log.warning(f"Ending trace which is not active: {self.trace_id}, contextvar_token not set")
            return

        self._in_progress = False
        self.queue_manager.report_trace_end(self)  # no-op
        Scope.reset_current_trace(self._contextvar_token)
        self._contextvar_token = None

        self.root_span.end()

    @override
    def flush(self, blocking: bool = True) -> None:
        self.root_span.flush(blocking=blocking)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from .._types import FileTypes

__all__ = ["BuildCreateParams"]


class BuildCreateParams(TypedDict, total=False):
    context_archive: Required[FileTypes]
    """
    tar.gz archive containing the build context (Dockerfile and any files needed for
    the build)
    """

    image_name: Required[str]
    """Name for the built image"""

    build_args: str
    """JSON string of build arguments"""

    image_tag: str
    """Tag for the built image"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel
from .shared.identity import Identity

__all__ = ["DatasetItem"]


class DatasetItem(BaseModel):
    id: str

    content_hash: str

    created_at: datetime

    created_by: Identity
    """The identity that created the entity."""

    data: object

    updated_at: datetime

    archived_at: Optional[datetime] = None

    dataset_id: Optional[str] = None

    files: Optional[Dict[str, str]] = None

    object: Optional[Literal["dataset.item"]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union
from typing_extensions import TypeAlias

from .inference_response import InferenceResponse
from .inference_response_chunk import InferenceResponseChunk

__all__ = ["InferenceCreateResponse"]

InferenceCreateResponse: TypeAlias = Union[InferenceResponse, InferenceResponseChunk]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["AssessmentType"]

AssessmentType: TypeAlias = Literal["comment", "rating", "approval", "rubric", "metadata", "overwrite"]
